"""A module that defines constants of expression's variable names.
"""

from typing_extensions import Final

DOCUMENT: Final[str] = 'document'
DISPLAY_OBJECT: Final[str] = 'do'
GRAPHICS: Final[str] = 'g'
RECTANGLE: Final[str] = 'rect'
SPRITE: Final[str] = 'sp'
ARRAY: Final[str] = 'arr'
DICTIONARY: Final[str] = 'dct'
INDEX: Final[str] = 'idx'
BOOLEAN: Final[str] = 'b'
INT: Final[str] = 'i'
NUMBER: Final[str] = 'n'
STRING: Final[str] = 's'
SNAPSHOT: Final[str] = 'snapshot'
ANY: Final[str] = 'any'
EVENT: Final[str] = 'evt'
MOUSE_EVENT: Final[str] = 'mevt'
WHEEL_EVENT: Final[str] = 'wevt'
